// FLS1.cpp: source code for FLS1, a saved game decrypter by Matthew 'IGx89' Lieder
// that uses Sherlog's decryption formula
//
// I used MSVC++ .Net to compile this, and used a few tricks I know to make the executable only 4kB :)
//

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <stdio.h>
#include <tchar.h>

int _tmain(int argc, _TCHAR* argv[])
{
	char input_filename[400],output_filename[400];
	FILE * f = NULL;
	int file_size;

	printf ("Freelancer saved game decrypter (FLS1) - Matthew 'IGx89' Lieder\nDecryption formula (the brains of this app) - Sherlog\n\n");
	if (argc < 2)
	{
		printf ("Usage is: fls1 (/newfile) savedgame.fl\n/newfile makes FLS1 output to savedgame.fl.txt instead of savedgame.fl\n\n");
		printf("Examples:\nfls1 autosave.fl\nfls1 /newfile autosave.fl\n");
		getchar();
		return 1;
	}

	if(stricmp(argv[1],"/newfile") == 0)
	{
		sprintf (input_filename, "%s", argv[2]);
		sprintf (output_filename, "%s.txt", argv[2]);
	}
	else
	{
		sprintf (input_filename, "%s", argv[1]);
		sprintf (output_filename, "%s", argv[1]);
	}

	f = fopen (input_filename, "rb");
	if (!f)
	{
		printf ("Error opening file %s\n", input_filename);
		getchar();
		return 2;
	}

	// get file size
	fseek(f, 0, SEEK_END);
	file_size = ftell(f);
	char *buffer = new char[file_size-3];
	fseek(f, 0, SEEK_SET);

	char header[5];
	if(fread(header, 4, 1, f) <= 0)
	{
		printf ("Error: unable to read data from %s!", input_filename);
		getchar();
		return 3;
	}
	header[4] = 0;
	if(strcmp(header,"FLS1") != 0)
	{
		printf ("Error: %s is not an encrypted saved game file!", input_filename);
		getchar();
		return 4;
	}

	if(fread(buffer, file_size-4, 1, f) <= 0)
	{
		printf ("Error: unable to read data from %s!", input_filename);
		getchar();
		return 5;
	}

	FILE * out = fopen (output_filename, "wb");
	if (!out)
	{
		printf ("Error: Cannot open output file %s\n", output_filename);
		getchar();
		return 6;
	}

	int i=0;
	unsigned char Key = 0;
	for(int i=0;i<file_size-4;i++)
	{
		switch(i%4)
		{
		case 0:
			Key = 'G';
			break;
		case 1:
			Key = 'e';
			break;
		case 2:
			Key = 'n';
			break;
		case 3:
			Key = 'e';
			break;
		}
		Key = (Key+i) % 256;
		buffer[i] = buffer[i] ^ (0x80 | Key);
	}

	if(fwrite(buffer, file_size-4, 1, out) <= 0)
	{
		printf ("Error: unable to write data to %s!", output_filename);
		getchar();
		return 7;
	}

	fclose(f);
	fclose(out);
	printf("%s successfully decrypted to %s!",input_filename,output_filename);
	getchar();
	return 0;
}